@@ -4,7 +4,11 @@ module BenchTime
   module Posix
     extend FFI::Library
     ffi_lib FFI::Library::LIBC
-    attach_function :time, [ :buffer_out ], :ulong, :ignore_error => true
+    if FFI::Library::LIBC =~ /ucrtbase/
+      attach_function :time, :_time64, [ :buffer_out ], :uint64, ignore_error: true
+    else
+      attach_function :time, [ :buffer_out ], :ulong, ignore_error: true
+    end
   end
 
   puts "Benchmark FFI time(3) with nil argument performance, #{ITER}x"
