@@ -18,7 +18,11 @@ old = Dir.pwd
 Dir.chdir(File.dirname(EXT))
 
 nul = File.open(File::NULL)
-make = system('type gmake', { :out => nul, :err => nul }) && 'gmake' || 'make'
+make = if RUBY_PLATFORM =~ /mswin/i
+  'nmake'
+else
+  system('type gmake', { :out => nul, :err => nul }) && 'gmake' || 'make'
+end
 
 # create Makefile
 system(RbConfig.ruby, "extconf.rb")
@@ -37,7 +41,7 @@ module LibWrap
   extend FFI::Library
   ffi_lib EXT
   callback :completion_function, [:string, :long, :uint8], :void
-  attach_function :do_work, [:pointer, :completion_function], :int
+  attach_function :do_work, [:string, :completion_function], :int
   Callback = Proc.new do |buf_ptr, count, code|
     puts "callback called with #{[buf_ptr, count, code].inspect}"
     nil
