@@ -26,14 +26,18 @@ module TestEnum3
   ffi_lib TestLibrary::PATH
 
   enum :enum_type1, [:c1, :c2, :c3, :c4]
-  enum :enum_type2, [:c5, 42, :c6, :c7, :c8]
-  enum :enum_type3, [:c9, 42, :c10, :c11, 4242, :c12]
-  enum :enum_type4, [:c13, 42, :c14, 4242, :c15, 424242, :c16, 42424242]
-
   attach_function :test_tagged_typedef_enum1, [:enum_type1], :enum_type1
+
+  enum :enum_type2, [:c5, 42, :c6, :c7, :c8]
   attach_function :test_tagged_typedef_enum2, [:enum_type2], :enum_type2
+
+  enum :enum_type3, [:c9, 42, :c10, :c11, 4242, :c12]
   attach_function :test_tagged_typedef_enum3, [:enum_type3], :enum_type3
+
+  enum :enum_type4, [:c13, 42, :c14, 4242, :c15, 424242, :c16, 42424242]
   attach_function :test_tagged_typedef_enum4, [:enum_type4], :enum_type4
+
+  freeze
 end
 
 module TestEnum4
@@ -44,18 +48,22 @@ module TestEnum4
   enum :enum_type1, [:c5, 0x42, :c6, :c7, :c8]
   enum :enum_type2, [:c9, 0x42, :c10, :c11, 0x4242, :c12]
   enum :enum_type3, [:c13, 0x42, :c14, 0x4242, :c15, 0x42424242, :c16, 0x4242424242424242]
-  enum FFI::Type::UINT16, :enum_type4, [:c17, 0x42, :c18, :c19, :c20]
-  enum FFI::Type::UINT32, :enum_type5, [:c21, 0x42, :c22, :c23, 0x4242, :c24]
-  enum FFI::Type::UINT64, :enum_type6, [:c25, 0x42, :c26, 0x4242, :c27, 0x42424242, :c28, 0x4242424242424242]
-  enum FFI::Type::UINT64, [:c29, 0x4242424242424242, :c30, :c31, :c32]
 
   attach_function :test_untagged_nonint_enum, [:uint8],  :uint8
   attach_function :test_tagged_nonint_enum1,  [:uint16], :uint16
   attach_function :test_tagged_nonint_enum2,  [:uint32], :uint32
   attach_function :test_tagged_nonint_enum3,  [:uint64], :uint64
+
+  enum FFI::Type::UINT16, :enum_type4, [:c17, 0x42, :c18, :c19, :c20]
+  enum FFI::Type::UINT32, :enum_type5, [:c21, 0x42, :c22, :c23, 0x4242, :c24]
+  enum FFI::Type::UINT64, :enum_type6, [:c25, 0x42, :c26, 0x4242, :c27, 0x42424242, :c28, 0x4242424242424242]
+  enum FFI::Type::UINT64, [:c29, 0x4242424242424242, :c30, :c31, :c32]
+
   attach_function :test_tagged_nonint_enum4, :test_tagged_nonint_enum1,  [:enum_type4], :enum_type4
   attach_function :test_tagged_nonint_enum5, :test_tagged_nonint_enum2,  [:enum_type5], :enum_type5
   attach_function :test_tagged_nonint_enum6, :test_tagged_nonint_enum3,  [:enum_type6], :enum_type6
+
+  freeze
 end
 
 describe "A library with no enum defined" do
@@ -430,4 +438,18 @@ describe "All enums" do
       end
     end.to raise_error(ArgumentError, /duplicate/)
   end
+
+  it "should be usable in Ractor", :ractor do
+    res = Ractor.new do
+      [
+        TestEnum1.test_untagged_enum(:c1),
+        TestEnum3.test_tagged_typedef_enum1(:c1),
+        TestEnum4.test_tagged_nonint_enum4(0x45),
+        TestEnum3.enum_type(:enum_type1)[0],
+        TestEnum4.enum_type(:enum_type6)[0x4242424242424242],
+        TestEnum4.enum_value(:c3)
+      ]
+    end.take
+    expect( res ).to eq( [0, :c1, :c20, :c1, :c28, 2] )
+  end
 end
