@@ -56,7 +56,7 @@ describe "Pointer" do
     expect(PointerTestLib.ptr_ret_int32_t(ptr, 0)).to eq(magic)
   end
 
-  it "Fixnum cannot be used as a Pointer argument" do
+  it "Integer cannot be used as a Pointer argument" do
     expect { PointerTestLib.ptr_ret_int32_t(0, 0) }.to raise_error(ArgumentError)
   end
 
@@ -76,6 +76,16 @@ describe "Pointer" do
     expect(FFI::Pointer::NULL.to_ptr).to eq(FFI::Pointer::NULL)
   end
 
+  it "equals itself" do
+    memory = FFI::MemoryPointer.new :pointer
+    expect(memory == memory).to be true
+  end
+
+  it "does not equal non pointers" do
+    memory = FFI::MemoryPointer.new :pointer
+    expect(memory == Hash.new).to be false
+  end
+
   describe "pointer type methods" do
 
     it "#read_pointer" do
@@ -150,6 +160,9 @@ describe "Pointer" do
     it 'returns true when compared with nil' do
       expect((FFI::Pointer::NULL == nil)).to be true
     end
+    it 'returns false when compared with a non-pointer object' do
+      expect((FFI::Pointer::NULL == Array.new)).to be false
+    end
     it 'should not raise an error when attempting read/write zero length array' do
       null_ptr = FFI::Pointer::NULL
       expect( null_ptr.read_array_of_uint(0) ).to eq([])
@@ -255,15 +268,15 @@ describe "Pointer" do
 
   describe "#inspect" do
     it "should include the address" do
-      FFI::Pointer.new(1234).inspect.should =~ /address=0x0*4d2/
+      expect(FFI::Pointer.new(1234).inspect).to match(/address=0x0*4d2/)
     end
 
     it "should not include the size if the pointer is unsized" do
-      FFI::Pointer.new(1234).inspect.should_not =~ /size=/
+      expect(FFI::Pointer.new(1234).inspect).not_to match(/size=/)
     end
 
     it "should include the size if there is one" do
-      FFI::MemoryPointer.new(:char, 16).inspect.should =~ /size=16/
+      expect(FFI::MemoryPointer.new(:char, 16).inspect).to match(/size=16/)
     end
   end
 end
